#!/usr/bin/python

import os
import math

#names of dir & file to output
outputDir = "output"
outputName = "icosa.svg"

#parameters of icosahedron
vertexHeight = [50.0, 68.182, 86.364, 104.545, 122.727, 140.909, 
                159.091, 177.273, 195.454, 213.636, 231.818, 250]
vertexMap = [[1,2],[1,3],[1,4],[1,5],[1,6],[2,3],[2,6],[2,7],[2,8],[3,4],
             [3,8],[3,9],[4,5],[4,9],[4,10],[5,6],[5,10],[5,11],[6,7],[6,11],
             [7,8],[7,11],[7,12],[8,9],[8,12],[9,10],[9,12],[10,11],[10,12],[11,12]]
slitPos = [14.0, 26.0, 33.0, 40.0, 49.0, 60.0, 
           52.0, 56.0, 50.0, 50.0, 50.0, 40.0]

#offset of origin point of Path
offsetPos = [10.0, 10.0]

#Material Thickness ( 0 < minimun vertex Height - 25 - mtrlThick / 2)
mtrlThick = 2.7

#depth of slit
slitDepth = 5.0

#Radius of Connector
connectorRad = 20.0

#function converting mm to num for Viewpoint value as str
def mm2vp(milim):
    return str(int(milim * 10)) 

#function rotating points 
def rotatePoint2d(point, rad):
    newX = point[0] * math.cos(rad) - point[1] * math.sin(rad)
    newY = point[0] * math.sin(rad) + point[1] * math.cos(rad)
    rotatedPoint = [newX, newY]

#create dir for output
if os.path.isdir(outputDir) != True:
    os.mkdir(outputDir)

#create output svg file
fo = open("./"+outputDir+"/"+outputName,"wb")

#svg start tag
fo.write('<?xml version="1.0" standalone="no"?>\n')
fo.write('<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">\n')
fo.write('<svg width="5000mm" height="500mm" viewBox="0 0 50000 5000" xmlns="http://www.w3.org/2000/svg" version="1.1">\n')

#parameter for merge
mergePos = 0.0
maxHeight = 0.0

for part in vertexMap:
    

    #calicurate points path go through
    arcRad = math.atan(2)
    horizonLegLen = vertexHeight[part[0]-1]
    angledLegLen = vertexHeight[part[1]-1]-slitPos[part[1]-1]-float(mtrlThick) / 2
    angledLegPoint = [angledLegLen * math.cos(arcRad) + offsetPos[0], angledLegLen * math.sin(arcRad) + offsetPos[1]]
    angledLegEdge = [vertexHeight[part[1]-1] * math.cos(arcRad) + offsetPos[0], vertexHeight[part[1]-1] * math.sin(arcRad) + offsetPos[1]]
    angledSlitWall = [float(slitDepth) * math.cos(arcRad - math.pi / 2), float(slitDepth) * math.sin(arcRad - math.pi / 2)]
    angledSlitBottom = [float(mtrlThick) * math.cos(arcRad), float(mtrlThick) * math.sin(arcRad)]
    medianPoint = [(offsetPos[0] * 2 + horizonLegLen + angledLegEdge[0]) / 3, (offsetPos[1] * 2 + angledLegEdge[1]) / 3]
    
    if angledLegEdge[1] > maxHeight:
        maxHeight = angledLegEdge[1]

    #record width of part
    mergePos += horizonLegLen
    
    #start mergin tag
    fo.write('<g transform="translate('+ mm2vp(mergePos) +')">')

    #Drawing - path start tag
    #fo.write('<path d="M' + mm2vp(float(offsetPos[0]) + horizonLegLen) + ',' + mm2vp(float(offsetPos[1])) + ' ')
    fo.write('<path d="M' + mm2vp(float(mtrlThick) / 2 - slitPos[part[0]-1] + horizonLegLen + offsetPos[0]) + ',' + mm2vp(float(offsetPos[1])) + ' ')
    
    #Draing - from edge to slit of horizontal leg
    #fo.write('l' + mm2vp(float(mtrlThick) / 2 - slitPos[part[0]-1]) + ',0 ')
    fo.write('l0,' + mm2vp(slitDepth) + ' l' + mm2vp(-mtrlThick) + ',0 l0,' + mm2vp(-slitDepth) + ' ')
    
    #Draing - inner curve
    #fo.write('Q' + mm2vp2(offsetPos[0]) + ',' + mm2vp2(offsetPos[1]) + ' ' + mm2vp2(angledLegPoint[0]) + ',' + mm2vp2(angledLegPoint[1]) + ' ')
    fo.write('C' + mm2vp(offsetPos[0]) + ',' + mm2vp(offsetPos[1]) + ' ' + mm2vp(offsetPos[0]) + ',' + mm2vp(offsetPos[1]) + ' ' 
             + mm2vp(angledLegPoint[0]) + ',' + mm2vp(angledLegPoint[1]) + ' ')

    #Drawing - from slit to edge of angled leg
    fo.write('l' + mm2vp(angledSlitWall[0]) + ',' + mm2vp(angledSlitWall[1]) + ' ')
    fo.write('l' + mm2vp(angledSlitBottom[0]) + ',' + mm2vp(angledSlitBottom[1]) + ' ')
    fo.write('l' + mm2vp(-angledSlitWall[0]) + ',' + mm2vp(-angledSlitWall[1]) + ' ')
    fo.write('L' + mm2vp(angledLegEdge[0]) + ',' + mm2vp(angledLegEdge[1]) + ' ')

    #Drawing - outer curve
#    fo.write('Q' + mm2vp(medianPoint[0]) + ',' + mm2vp(medianPoint[1]) + ' ' + mm2vp(horizonLegLen + offsetPos[0]) + ',' + mm2vp(offsetPos[1]) + ' ')
    fo.write('C' + mm2vp(medianPoint[0]) + ',' + mm2vp(medianPoint[1]) + ' ' + mm2vp(medianPoint[0]) + ',' + mm2vp(medianPoint[1]) + ' '
             + mm2vp(horizonLegLen + offsetPos[0]) + ',' + mm2vp(offsetPos[1]) + ' ')

    #Drawing - path end tag
    fo.write('Z" fill="none" stroke="red" stroke-width="1"  />\n') 

    #Draing - add label
    fo.write('<text class="label" font-size="22" x="'+  mm2vp(horizonLegLen - float(mtrlThick)/2 - slitPos[part[0]-1] + offsetPos[0] - 3)
             +'" y= "'+ mm2vp(offsetPos[0] +3 ) +'">' + str(part[0]) +'</text>\n')
    fo.write('<text class="label" font-size="22" x="' + mm2vp(angledLegPoint[0]) +'" y= "' + mm2vp(angledLegPoint[1] - 3) +'">'+ str(part[1]) +'</text>\n')

    #end mergin tag
    fo.write('</g>')

#Drawing - connectors
rPoints = [[-float(mtrlThick) / 2, connectorRad],[-float(mtrlThick) / 2, connectorRad - slitDepth],
                    [float(mtrlThick) / 2, connectorRad - slitDepth],[float(mtrlThick) / 2, connectorRad]]

for connectorNum in range(15):
    #fo.write('<g transform="translate('+ mm2vp(mergePos + offsetPos[0]) + ',' + mm2vp((connectorRad * 2 + offsetPos[1])*connectorNum) +')">')
    fo.write('<g transform="translate('+ mm2vp((connectorRad * 2 + 3)*connectorNum + connectorRad * 2 + offsetPos[1]) 
             + ', ' + mm2vp(maxHeight + offsetPos[1]*4)  +')">')
    
    #Drawing - connectors
    for numR in range(5):
        if numR == 0:
            fo.write('<path d="M' + mm2vp(rPoints[0][0]) + ',' + mm2vp(rPoints[0][1]) + ' ')
        else:
            fo.write('L' + mm2vp(rPoints[0][0]) +','+ mm2vp(rPoints[0][1]) + ' ')

        for numP in range(1,4):
            fo.write('L' + mm2vp(rPoints[numP][0]) +','+ mm2vp(rPoints[numP][1]) + ' ')

        if numR == 4:
            #Drawing - path end tag
            fo.write('Z" fill="none" stroke="red" stroke-width="1"  />\n')
        else:
            for numT in range(4):
                #rotate rPoints
                rad = -2 * math.pi / 5
                newX = rPoints[numT][0] * math.cos(rad) - rPoints[numT][1] * math.sin(rad)
                newY = rPoints[numT][0] * math.sin(rad) + rPoints[numT][1] * math.cos(rad)
                rPoints[numT][0] = newX
                rPoints[numT][1] = newY
                """
                rPoints[numT] = rotatePoint2d(rPoints[numT], math.pi/5)
                print numT
                """
    fo.write('</g>\n')

#svg end tag
fo.write('</svg>\n')
